#
#    (C) Quantum Computing Inc., 2020.
#
#    THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE of the Copyright Holder. 
#    The contents of this file may not be disclosed to third parties, copied 
#    or duplicated in any form, in whole or in part, without the prior written
#    permission of the Copyright Holder.
#
import base64

from typing import Any
import numpy as np
import scipy.sparse as sp


def encode_data(data):
    if isinstance(data, sp.spmatrix):
        return encode_sparse_array(data)
    elif isinstance(data, dict):
        return encode_dict(data)
    else:
        return base64.b64encode(data).decode('utf-8')


def encode_dict(data):
    qubo_str = str(data).encode('utf-8')
    return encode_data(qubo_str)


def encode_ndarray(qubo: np.ndarray) -> Any:
    # Enforce float type so we can assume this on the server side
    qubo = qubo.astype(np.float)
    qubo_str = qubo.tostring()
    return encode_data(qubo_str)


def encode_sparse_array(qubo: sp.spmatrix) -> dict:
    # COO is easy to encode as three separate matrices: data, row, col
    # Enforce float type so we can assume this on the server side
    qubo = qubo.tocoo().astype(np.float)
    qubo_data_encoded = encode_data(qubo.data.tostring())
    qubo_row_encoded = encode_data(qubo.row.tostring())
    qubo_col_encoded = encode_data(qubo.col.tostring())
    return {'data': qubo_data_encoded, 'row': qubo_row_encoded, 'col': qubo_col_encoded}

